import pm4py
from pm4py.algo.discovery.dfg import algorithm as dfg_discovery
from pm4py.statistics.start_activities.log import get as start_activities_module
from pm4py.statistics.end_activities.log import get as end_activities_module
from pm4py.util import constants
import pandas as pd
import numpy as np

def mine_process(workflow_df, case_name, concept_name, time, end_time=None, transition='frequency'):
    if end_time == None:
        event_log = pm4py.format_dataframe(workflow_df, case_id=case_name, activity_key=concept_name, timestamp_key=time)
    else:
        event_log = pm4py.format_dataframe(workflow_df, case_id=case_name, activity_key=concept_name, start_timestamp_key=time, timestamp_key=end_time)
    
    if transition=='frequency':
        variant = dfg_discovery.Variants.FREQUENCY
    elif transition=='performance':
        variant = dfg_discovery.Variants.PERFORMANCE
        
    dfg = dfg_discovery.apply(event_log, variant=variant)
    
    grouped_df = event_log.groupby('case:concept:name')
    last_eve_df = grouped_df.last()
    endact_dict = dict(last_eve_df['concept:name'].value_counts())
    
    first_eve_df = grouped_df.first()
    startact_dict = dict(first_eve_df['concept:name'].value_counts())
    
    dfg_df = pd.DataFrame([(k[0], k[1], dfg[k]) for k in dfg], columns=['source', 'target', 'weight'])
    if end_time == None and transition=='performance':
        dfg_freq = dfg_discovery.apply(event_log, variant=dfg_discovery.Variants.FREQUENCY)
        dfg_freq_df = pd.DataFrame([(k[0], k[1], dfg_freq[k]) for k in dfg_freq], columns=['source', 'target', 'freq'])
        dfg_weighted_df = pd.merge(dfg_df, dfg_freq_df, on=['source', 'target'])
        weighted_average = lambda x: np.average(x, weights=dfg_freq_df.loc[x.index, "freq"])
        activities_count_df = dfg_df.groupby('source').agg(weighted_time=('weight', weighted_average)).reset_index()
        activities_count_df.columns = ['step', 'frequency']
        activities_count_df = activities_count_df.append(pd.DataFrame({'step': ['END'], 'frequency': [0]}))
    elif transition=='frequency':
        activities_count = pm4py.get_event_attribute_values(event_log, 'concept:name')
        activities_count_df = pd.DataFrame([(k, activities_count[k]) for k in activities_count], columns=['step', 'frequency'])
    else:
        activities_count_df = workflow_df.groupby(concept_name).agg({'process_time': 'mean'}).reset_index()
        activities_count_df.columns = ['step', 'frequency']
    
    start_end_mined_df = pd.DataFrame([(k, startact_dict[k], 'start') for k in startact_dict], 
                                     columns=['step', 'frequency', 'start_end']).append(
        pd.DataFrame([(k, endact_dict[k], 'end') for k in endact_dict], 
                                     columns=['step', 'frequency', 'start_end']))
    
    return dfg_df, start_end_mined_df, activities_count_df
